@@ -1,5 +1,5 @@
+use crate::error::S3Error;
 use crate::{Bucket, Region};
-use anyhow::Result;
 
 /// [AWS Documentation](https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#CannedACL)
 #[allow(dead_code)]
@@ -119,7 +119,7 @@ impl BucketConfiguration {
         }
     }
 
-    pub fn add_headers(&self, headers: &mut HeaderMap) -> Result<()> {
+    pub fn add_headers(&self, headers: &mut HeaderMap) -> Result<(), S3Error> {
         headers.insert(
             HeaderName::from_static("x-amz-acl"),
             self.acl.to_string().parse().unwrap(),
