@@ -1,14 +1,14 @@
-use chrono::{DateTime, Utc};
 use hmac::Mac;
-use hmac::NewMac;
+use std::collections::HashMap;
+use time::format_description::well_known::Rfc2822;
+use time::OffsetDateTime;
 use url::Url;
 
 use crate::bucket::Bucket;
 use crate::command::Command;
+use crate::error::S3Error;
 use crate::signing;
-use crate::LONG_DATE;
-use anyhow::anyhow;
-use anyhow::Result;
+use crate::LONG_DATETIME;
 use http::header::{
     HeaderName, ACCEPT, AUTHORIZATION, CONTENT_LENGTH, CONTENT_TYPE, DATE, HOST, RANGE,
 };
@@ -19,27 +19,30 @@ pub trait Request {
     type Response;
     type HeaderMap;
 
-    async fn response(&self) -> Result<Self::Response>;
-    async fn response_data(&self, etag: bool) -> Result<(Vec<u8>, u16)>;
+    async fn response(&self) -> Result<Self::Response, S3Error>;
+    async fn response_data(&self, etag: bool) -> Result<(Vec<u8>, u16), S3Error>;
     #[cfg(feature = "with-tokio")]
     async fn response_data_to_writer<T: tokio::io::AsyncWrite + Send + Unpin>(
         &self,
         writer: &mut T,
-    ) -> Result<u16>;
+    ) -> Result<u16, S3Error>;
     #[cfg(feature = "with-async-std")]
     async fn response_data_to_writer<T: futures_io::AsyncWrite + Send + Unpin>(
         &self,
         writer: &mut T,
-    ) -> Result<u16>;
+    ) -> Result<u16, S3Error>;
     #[cfg(feature = "sync")]
-    fn response_data_to_writer<T: std::io::Write + Send>(&self, writer: &mut T) -> Result<u16>;
-    async fn response_header(&self) -> Result<(Self::HeaderMap, u16)>;
-    fn datetime(&self) -> DateTime<Utc>;
+    fn response_data_to_writer<T: std::io::Write + Send>(
+        &self,
+        writer: &mut T,
+    ) -> Result<u16, S3Error>;
+    async fn response_header(&self) -> Result<(Self::HeaderMap, u16), S3Error>;
+    fn datetime(&self) -> OffsetDateTime;
     fn bucket(&self) -> Bucket;
     fn command(&self) -> Command;
     fn path(&self) -> String;
 
-    fn signing_key(&self) -> Result<Vec<u8>> {
+    fn signing_key(&self) -> Result<Vec<u8>, S3Error> {
         signing::signing_key(
             &self.datetime(),
             &self
@@ -74,7 +77,7 @@ pub trait Request {
     }
 
     fn long_date(&self) -> String {
-        self.datetime().format(LONG_DATE).to_string()
+        self.datetime().format(LONG_DATETIME).unwrap()
     }
 
     fn string_to_sign(&self, request: &str) -> String {
@@ -85,34 +88,31 @@ pub trait Request {
         self.bucket().host()
     }
 
-    fn presigned(&self) -> Result<String> {
-        let expiry = match self.command() {
-            Command::PresignGet { expiry_secs } => expiry_secs,
-            Command::PresignPut { expiry_secs, .. } => expiry_secs,
-            Command::PresignDelete { expiry_secs } => expiry_secs,
+    fn presigned(&self) -> Result<String, S3Error> {
+        let (expiry, custom_headers, custom_queries) = match self.command() {
+            Command::PresignGet {
+                expiry_secs,
+                custom_queries,
+            } => (expiry_secs, None, custom_queries),
+            Command::PresignPut {
+                expiry_secs,
+                custom_headers,
+            } => (expiry_secs, custom_headers, None),
+            Command::PresignDelete { expiry_secs } => (expiry_secs, None, None),
             _ => unreachable!(),
         };
 
-        #[allow(clippy::collapsible_match)]
-        if let Command::PresignPut { custom_headers, .. } = self.command() {
-            if let Some(custom_headers) = custom_headers {
-                let authorization = self.presigned_authorization(Some(&custom_headers))?;
-                return Ok(format!(
-                    "{}&X-Amz-Signature={}",
-                    self.presigned_url_no_sig(expiry, Some(&custom_headers))?,
-                    authorization
-                ));
-            }
-        }
-
         Ok(format!(
             "{}&X-Amz-Signature={}",
-            self.presigned_url_no_sig(expiry, None)?,
-            self.presigned_authorization(None)?
+            self.presigned_url_no_sig(expiry, custom_headers.as_ref(), custom_queries.as_ref())?,
+            self.presigned_authorization(custom_headers.as_ref())?
         ))
     }
 
-    fn presigned_authorization(&self, custom_headers: Option<&HeaderMap>) -> Result<String> {
+    fn presigned_authorization(
+        &self,
+        custom_headers: Option<&HeaderMap>,
+    ) -> Result<String, S3Error> {
         let mut headers = HeaderMap::new();
         let host_header = self.host_header();
         headers.insert(HOST, host_header.parse().unwrap());
@@ -123,43 +123,41 @@ pub trait Request {
         }
         let canonical_request = self.presigned_canonical_request(&headers)?;
         let string_to_sign = self.string_to_sign(&canonical_request);
-        let mut hmac = signing::HmacSha256::new_from_slice(&self.signing_key()?)
-            .map_err(|e| anyhow! {"{}",e})?;
+        let mut hmac = signing::HmacSha256::new_from_slice(&self.signing_key()?)?;
         hmac.update(string_to_sign.as_bytes());
         let signature = hex::encode(hmac.finalize().into_bytes());
         // let signed_header = signing::signed_header_string(&headers);
         Ok(signature)
     }
 
-    fn presigned_canonical_request(&self, headers: &HeaderMap) -> Result<String> {
-        let expiry = match self.command() {
-            Command::PresignGet { expiry_secs } => expiry_secs,
-            Command::PresignPut { expiry_secs, .. } => expiry_secs,
-            Command::PresignDelete { expiry_secs } => expiry_secs,
+    fn presigned_canonical_request(&self, headers: &HeaderMap) -> Result<String, S3Error> {
+        let (expiry, custom_headers, custom_queries) = match self.command() {
+            Command::PresignGet {
+                expiry_secs,
+                custom_queries,
+            } => (expiry_secs, None, custom_queries),
+            Command::PresignPut {
+                expiry_secs,
+                custom_headers,
+            } => (expiry_secs, custom_headers, None),
+            Command::PresignDelete { expiry_secs } => (expiry_secs, None, None),
             _ => unreachable!(),
         };
 
-        #[allow(clippy::collapsible_match)]
-        if let Command::PresignPut { custom_headers, .. } = self.command() {
-            if let Some(custom_headers) = custom_headers {
-                return Ok(signing::canonical_request(
-                    &self.command().http_verb().to_string(),
-                    &self.presigned_url_no_sig(expiry, Some(&custom_headers))?,
-                    headers,
-                    "UNSIGNED-PAYLOAD",
-                ));
-            }
-        }
-
         Ok(signing::canonical_request(
             &self.command().http_verb().to_string(),
-            &self.presigned_url_no_sig(expiry, None)?,
+            &self.presigned_url_no_sig(expiry, custom_headers.as_ref(), custom_queries.as_ref())?,
             headers,
             "UNSIGNED-PAYLOAD",
         ))
     }
 
-    fn presigned_url_no_sig(&self, expiry: u32, custom_headers: Option<&HeaderMap>) -> Result<Url> {
+    fn presigned_url_no_sig(
+        &self,
+        expiry: u32,
+        custom_headers: Option<&HeaderMap>,
+        custom_queries: Option<&HashMap<String, String>>,
+    ) -> Result<Url, S3Error> {
         let bucket = self.bucket();
         let token = if let Some(security_token) = bucket.security_token() {
             Some(security_token)
@@ -167,7 +165,7 @@ pub trait Request {
             bucket.session_token()
         };
         let url = Url::parse(&format!(
-            "{}{}",
+            "{}{}{}",
             self.url(),
             &signing::authorization_query_params_no_sig(
                 &self.bucket().access_key().unwrap(),
@@ -176,7 +174,8 @@ pub trait Request {
                 expiry,
                 custom_headers,
                 token
-            )?
+            )?,
+            &signing::flatten_queries(custom_queries),
         ))?;
 
         Ok(url)
@@ -196,8 +195,7 @@ pub trait Request {
         };
 
         url_str.push('/');
-
-        url_str.push_str(&signing::uri_encode(&path, true));
+        url_str.push_str(&signing::uri_encode(&path, false));
 
         // Append to url_path
         #[allow(clippy::collapsible_match)]
@@ -312,11 +310,10 @@ pub trait Request {
         )
     }
 
-    fn authorization(&self, headers: &HeaderMap) -> Result<String> {
+    fn authorization(&self, headers: &HeaderMap) -> Result<String, S3Error> {
         let canonical_request = self.canonical_request(headers);
         let string_to_sign = self.string_to_sign(&canonical_request);
-        let mut hmac = signing::HmacSha256::new_from_slice(&self.signing_key()?)
-            .map_err(|e| anyhow! {"{}",e})?;
+        let mut hmac = signing::HmacSha256::new_from_slice(&self.signing_key()?)?;
         hmac.update(string_to_sign.as_bytes());
         let signature = hex::encode(hmac.finalize().into_bytes());
         let signed_header = signing::signed_header_string(headers);
@@ -329,7 +326,7 @@ pub trait Request {
         ))
     }
 
-    fn headers(&self) -> Result<HeaderMap> {
+    fn headers(&self) -> Result<HeaderMap, S3Error> {
         // Generate this once, but it's used in more than one place.
         let sha256 = self.command().sha256();
 
@@ -443,7 +440,10 @@ pub trait Request {
         // range and can't be used again e.g. reply attacks. Adding this header
         // after the generation of the Authorization header leaves it out of
         // the signed headers.
-        headers.insert(DATE, self.datetime().to_rfc2822().parse().unwrap());
+        headers.insert(
+            DATE,
+            self.datetime().format(&Rfc2822).unwrap().parse().unwrap(),
+        );
 
         Ok(headers)
     }
