@@ -2,21 +2,22 @@
 //!
 //! [link]: https://docs.aws.amazon.com/AmazonS3/latest/API/sig-v4-authenticating-requests.html
 
+use std::collections::HashMap;
 use std::str;
 
-use chrono::{DateTime, Utc};
-use hmac::{Hmac, Mac, NewMac};
+use hmac::{Hmac, Mac};
+use http::HeaderMap;
 use percent_encoding::{utf8_percent_encode, AsciiSet, CONTROLS};
 use sha2::{Digest, Sha256};
+use time::{macros::format_description, OffsetDateTime};
 use url::Url;
 
+use crate::error::S3Error;
 use crate::region::Region;
-use anyhow::anyhow;
-use anyhow::Result;
-use http::HeaderMap;
+use crate::LONG_DATETIME;
 
-const SHORT_DATE: &str = "%Y%m%d";
-const LONG_DATETIME: &str = "%Y%m%dT%H%M%SZ";
+const SHORT_DATE: &[time::format_description::FormatItem<'static>] =
+    format_description!("[year][month][day]");
 
 pub type HmacSha256 = Hmac<Sha256>;
 
@@ -56,7 +57,6 @@ pub const FRAGMENT: &AsciiSet = &CONTROLS
 pub const FRAGMENT_SLASH: &AsciiSet = &FRAGMENT.add(b'/');
 
 /// Encode a URI following the specific requirements of the AWS service.
-#[cfg(not(feature = "never-encode-slash"))]
 pub fn uri_encode(string: &str, encode_slash: bool) -> String {
     if encode_slash {
         utf8_percent_encode(string, FRAGMENT_SLASH).to_string()
@@ -65,11 +65,6 @@ pub fn uri_encode(string: &str, encode_slash: bool) -> String {
     }
 }
 
-#[cfg(feature = "never-encode-slash")]
-pub fn uri_encode(string: &str, encode_slash: bool) -> String {
-    utf8_percent_encode(string, FRAGMENT).to_string()
-}
-
 /// Generate a canonical URI string from the given URL.
 pub fn canonical_uri_string(uri: &Url) -> String {
     // decode `Url`'s percent-encoding and then reencode it
@@ -130,22 +125,22 @@ pub fn canonical_request(method: &str, url: &Url, headers: &HeaderMap, sha256: &
 }
 
 /// Generate an AWS scope string.
-pub fn scope_string(datetime: &DateTime<Utc>, region: &Region) -> String {
+pub fn scope_string(datetime: &OffsetDateTime, region: &Region) -> String {
     format!(
         "{date}/{region}/s3/aws4_request",
-        date = datetime.format(SHORT_DATE),
+        date = datetime.format(SHORT_DATE).unwrap(),
         region = region
     )
 }
 
 /// Generate the "string to sign" - the value to which the HMAC signing is
 /// applied to sign requests.
-pub fn string_to_sign(datetime: &DateTime<Utc>, region: &Region, canonical_req: &str) -> String {
+pub fn string_to_sign(datetime: &OffsetDateTime, region: &Region, canonical_req: &str) -> String {
     let mut hasher = Sha256::default();
     hasher.update(canonical_req.as_bytes());
     let string_to = format!(
         "AWS4-HMAC-SHA256\n{timestamp}\n{scope}\n{hash}",
-        timestamp = datetime.format(LONG_DATETIME),
+        timestamp = datetime.format(LONG_DATETIME).unwrap(),
         scope = scope_string(datetime, region),
         hash = hex::encode(hasher.finalize().as_slice())
     );
@@ -155,23 +150,19 @@ pub fn string_to_sign(datetime: &DateTime<Utc>, region: &Region, canonical_req:
 /// Generate the AWS signing key, derived from the secret key, date, region,
 /// and service name.
 pub fn signing_key(
-    datetime: &DateTime<Utc>,
+    datetime: &OffsetDateTime,
     secret_key: &str,
     region: &Region,
     service: &str,
-) -> Result<Vec<u8>> {
+) -> Result<Vec<u8>, S3Error> {
     let secret = format!("AWS4{}", secret_key);
-    let mut date_hmac =
-        HmacSha256::new_from_slice(secret.as_bytes()).map_err(|e| anyhow! {"{}",e})?;
-    date_hmac.update(datetime.format(SHORT_DATE).to_string().as_bytes());
-    let mut region_hmac = HmacSha256::new_from_slice(&date_hmac.finalize().into_bytes())
-        .map_err(|e| anyhow! {"{}",e})?;
+    let mut date_hmac = HmacSha256::new_from_slice(secret.as_bytes())?;
+    date_hmac.update(datetime.format(SHORT_DATE).unwrap().as_bytes());
+    let mut region_hmac = HmacSha256::new_from_slice(&date_hmac.finalize().into_bytes())?;
     region_hmac.update(region.to_string().as_bytes());
-    let mut service_hmac = HmacSha256::new_from_slice(&region_hmac.finalize().into_bytes())
-        .map_err(|e| anyhow! {"{}",e})?;
+    let mut service_hmac = HmacSha256::new_from_slice(&region_hmac.finalize().into_bytes())?;
     service_hmac.update(service.as_bytes());
-    let mut signing_hmac = HmacSha256::new_from_slice(&service_hmac.finalize().into_bytes())
-        .map_err(|e| anyhow! {"{}",e})?;
+    let mut signing_hmac = HmacSha256::new_from_slice(&service_hmac.finalize().into_bytes())?;
     signing_hmac.update(b"aws4_request");
     Ok(signing_hmac.finalize().into_bytes().to_vec())
 }
@@ -179,7 +170,7 @@ pub fn signing_key(
 /// Generate the AWS authorization header.
 pub fn authorization_header(
     access_key: &str,
-    datetime: &DateTime<Utc>,
+    datetime: &OffsetDateTime,
     region: &Region,
     signed_headers: &str,
     signature: &str,
@@ -196,12 +187,12 @@ pub fn authorization_header(
 
 pub fn authorization_query_params_no_sig(
     access_key: &str,
-    datetime: &DateTime<Utc>,
+    datetime: &OffsetDateTime,
     region: &Region,
     expires: u32,
     custom_headers: Option<&HeaderMap>,
     token: Option<&str>,
-) -> Result<String> {
+) -> Result<String, S3Error> {
     let credentials = uri_encode(
         &format!("{}/{}", access_key, scope_string(datetime, region)),
         true,
@@ -224,7 +215,7 @@ pub fn authorization_query_params_no_sig(
             &X-Amz-Expires={expires}\
             &X-Amz-SignedHeaders={signed_headers}",
         credentials = credentials,
-        long_date = datetime.format(LONG_DATETIME),
+        long_date = datetime.format(LONG_DATETIME).unwrap(),
         expires = expires,
         signed_headers = signed_headers_string
     );
@@ -239,18 +230,36 @@ pub fn authorization_query_params_no_sig(
     Ok(query_params)
 }
 
+pub fn flatten_queries(queries: Option<&HashMap<String, String>>) -> String {
+    match queries {
+        None => String::new(),
+        Some(queries) => {
+            let mut query_str = String::new();
+            for (k, v) in queries {
+                query_str.push('&');
+                query_str.push_str(&uri_encode(k, true));
+                query_str.push('=');
+                query_str.push_str(&uri_encode(v, true));
+            }
+            query_str
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
-    use chrono::{TimeZone, Utc};
+    use std::convert::TryInto;
     use std::str;
-    use url::Url;
 
-    use super::*;
-
-    use crate::serde_types::ListBucketResult;
     use http::header::{HeaderName, HOST, RANGE};
     use http::HeaderMap;
     use serde_xml_rs as serde_xml;
+    use time::Date;
+    use url::Url;
+
+    use crate::serde_types::ListBucketResult;
+
+    use super::*;
 
     #[test]
     fn test_base_url_encode() {
@@ -333,7 +342,11 @@ mod tests {
     fn test_aws_signing_key() {
         let key = "wJalrXUtnFEMI/K7MDENG+bPxRfiCYEXAMPLEKEY";
         let expected = "c4afb1cc5771d871763a393e44b703571b55cc28424d1a5e86da6ed3c154a4b9";
-        let datetime = Utc.ymd(2015, 8, 30).and_hms(0, 0, 0);
+        let datetime = Date::from_calendar_date(2015, 8.try_into().unwrap(), 30)
+            .unwrap()
+            .with_hms(0, 0, 0)
+            .unwrap()
+            .assume_utc();
         let signature = signing_key(&datetime, key, &"us-east-1".parse().unwrap(), "iam").unwrap();
         assert_eq!(expected, hex::encode(signature));
     }
@@ -378,7 +391,11 @@ mod tests {
         let canonical = canonical_request("GET", &url, &headers, EXPECTED_SHA);
         assert_eq!(EXPECTED_CANONICAL_REQUEST, canonical);
 
-        let datetime = Utc.ymd(2013, 5, 24).and_hms(0, 0, 0);
+        let datetime = Date::from_calendar_date(2013, 5.try_into().unwrap(), 24)
+            .unwrap()
+            .with_hms(0, 0, 0)
+            .unwrap()
+            .assume_utc();
         let string_to_sign = string_to_sign(&datetime, &"us-east-1".parse().unwrap(), &canonical);
         assert_eq!(EXPECTED_STRING_TO_SIGN, string_to_sign);
 
