@@ -1,7 +1,7 @@
 use std::str::FromStr;
 
+use crate::error::S3Error;
 use crate::{bucket::CHUNK_SIZE, serde_types::HeadObjectResult};
-use anyhow::Result;
 
 #[cfg(feature = "with-async-std")]
 use async_std::fs::File;
@@ -36,7 +36,7 @@ use tokio::io::{AsyncRead, AsyncReadExt};
 /// }
 /// ```
 #[cfg(any(feature = "with-tokio", feature = "with-async-std"))]
-pub async fn etag_for_path(path: impl AsRef<Path>) -> Result<String> {
+pub async fn etag_for_path(path: impl AsRef<Path>) -> Result<String, S3Error> {
     let mut file = File::open(path).await?;
     let mut digests = Vec::new();
     let mut chunks = 0;
@@ -67,7 +67,7 @@ pub async fn etag_for_path(path: impl AsRef<Path>) -> Result<String> {
 /// println!("{}", etag);
 /// ```
 #[cfg(feature = "sync")]
-pub fn etag_for_path(path: impl AsRef<Path>) -> Result<String> {
+pub fn etag_for_path(path: impl AsRef<Path>) -> Result<String, S3Error> {
     let mut file = File::open(path)?;
     let mut digests = Vec::new();
     let mut chunks = 0;
@@ -90,7 +90,7 @@ pub fn etag_for_path(path: impl AsRef<Path>) -> Result<String> {
 }
 
 #[cfg(any(feature = "with-tokio", feature = "with-async-std"))]
-pub async fn read_chunk<R: AsyncRead + Unpin>(reader: &mut R) -> Result<Vec<u8>> {
+pub async fn read_chunk<R: AsyncRead + Unpin>(reader: &mut R) -> Result<Vec<u8>, S3Error> {
     let mut chunk = Vec::with_capacity(CHUNK_SIZE);
     let mut take = reader.take(CHUNK_SIZE as u64);
     take.read_to_end(&mut chunk).await?;
@@ -99,7 +99,7 @@ pub async fn read_chunk<R: AsyncRead + Unpin>(reader: &mut R) -> Result<Vec<u8>>
 }
 
 #[cfg(feature = "sync")]
-pub fn read_chunk<R: Read>(reader: &mut R) -> Result<Vec<u8>> {
+pub fn read_chunk<R: Read>(reader: &mut R) -> Result<Vec<u8>, S3Error> {
     let mut chunk = Vec::with_capacity(CHUNK_SIZE);
     let mut take = reader.take(CHUNK_SIZE as u64);
     take.read_to_end(&mut chunk)?;
@@ -171,6 +171,11 @@ impl From<&http::HeaderMap> for HeadObjectResult {
     }
 }
 
+pub(crate) fn error_from_response_data(data: Vec<u8>, code: u16) -> Result<S3Error, S3Error> {
+    let utf8_content = String::from_utf8(data)?;
+    Err(S3Error::Http(code, utf8_content))
+}
+
 #[cfg(test)]
 mod test {
     use crate::utils::etag_for_path;
